@@ -12,6 +12,8 @@ our %Transport = (
     'thrift'   => 'ElasticSearch::Transport::Thrift'
 );
 
+our $Skip_Log;
+
 #===================================
 sub new {
 #===================================
@@ -27,16 +29,22 @@ sub new {
 
     eval "require  $transport_class" or $class->throw( "Internal", $@ );
 
-    my $self = bless { _JSON => JSON->new(), _timeout => 120 },
+    my $self = bless {
+        _JSON         => JSON->new(),
+        _timeout      => 120,
+        _max_requests => 10_000
+        },
         $transport_class;
 
     my $servers = delete $params->{servers}
         or $self->throw( 'Param', 'No servers passed to new' );
 
-    $servers = $self->servers($servers);
-    $self->{_default_servers} = [@$servers];
-    if ( exists $params->{timeout} ) {
-        $self->timeout( delete $params->{timeout} );
+    $self->{_default_servers}
+        = [ ref $servers eq 'ARRAY' ? @$servers : $servers ];
+
+    for (qw(timeout max_requests)) {
+        next unless exists $params->{$_};
+        $self->$_( delete $params->{$_} );
     }
     $self->init($params);
     return $self;
@@ -71,7 +79,7 @@ ATTEMPT:
     while (1) {
         my $server = $single_server || $self->next_server;
 
-        $self->log_request( $server, $args );
+        $self->log_request( $server, $args ) unless $Skip_Log;
 
         $response_json = eval { $self->send_request( $server, $args ) }
             and last ATTEMPT;
@@ -97,13 +105,16 @@ ATTEMPT:
                     delete $error->{-vars}{content};
                 }
             }
+            return
+                if $error->isa('ElasticSearch::Error::Missing')
+                    && $args->{qs}{ignore_missing};
             die $error;
         }
         $self->throw( 'Request', $error, { request => $params } );
     }
 
     my $result = $json->decode($response_json);
-    $self->log_response( $result || $response_json );
+    $self->log_response( $result || $response_json ) unless $Skip_Log;
     return $result;
 }
 
@@ -112,6 +123,7 @@ sub refresh_servers {
 #===================================
     my $self = shift;
 
+    $self->{_refresh_in} = 0;
     delete $self->{_current_server};
 
     my %servers = map { $_ => 1 }
@@ -120,6 +132,8 @@ sub refresh_servers {
     my @all_servers = keys %servers;
     my $protocol    = $self->protocol;
 
+    local $Skip_Log = 1;
+
     foreach my $server (@all_servers) {
         next unless $server;
 
@@ -136,6 +150,7 @@ sub refresh_servers {
             } values %{ $nodes->{nodes} };
         next unless @servers;
 
+        $self->{_refresh_in} = $self->max_requests - 1;
         return $self->servers( \@servers );
     }
 
@@ -149,9 +164,13 @@ sub refresh_servers {
 #===================================
 sub next_server {
 #===================================
-    my $self    = shift;
+    my $self = shift;
+    $self->refresh_servers
+        unless $self->{_refresh_in}--;
+
     my @servers = @{ $self->servers };
-    my $next    = shift @servers;
+    my $next    = shift(@servers);
+
     $self->{_current_server} = { $$ => $next };
     $self->servers( @servers, $next );
     return $next;
@@ -171,7 +190,17 @@ sub servers {
     if (@_) {
         $self->{_servers} = ref $_[0] eq 'ARRAY' ? shift : [@_];
     }
-    return $self->{_servers} || [];
+    return $self->{_servers} ||= [];
+}
+
+#===================================
+sub max_requests {
+#===================================
+    my $self = shift;
+    if (@_) {
+        $self->{_max_requests} = shift;
+    }
+    return $self->{_max_requests} || 0;
 }
 
 #===================================
@@ -242,7 +271,7 @@ sub log_request {
     my $params = shift;
 
     my $data = $params->{data};
-    if ( defined $data ) {
+    if ( defined $data and $data ne "{}\n" ) {
         $data =~ s/'/\\u0027/g;
         $data = " -d '\n${data}'";
     }
@@ -255,7 +284,8 @@ sub log_request {
         scalar localtime(),
         $self->protocol, ${server}
     );
-    my $uri = $self->http_uri( '127.0.0.1:9200', @{$params}{ 'cmd', 'qs' } );
+    my %qs = ( %{ $params->{qs} }, pretty => 1 );
+    my $uri = $self->http_uri( '127.0.0.1:9200', $params->{cmd}, \%qs );
 
     my $method = $params->{method};
     print $log "curl -X$method '$uri' ${data}\n\n";
@@ -344,6 +374,10 @@ with the ElasticSearch server.
 It handles failover to the next node in case the current node closes the
 connection. All requests are round-robin'ed to all live servers.
 
+On the first request and every C<max_requests> after that (default 10,000),
+the list of live nodes is automatically refreshed.  This can be disabled
+by setting C<max_requests> to C<0>.
+
 Currently, the available backends are:
 
 =over
@@ -383,6 +417,7 @@ happens via the main L<ElasticSearch> class.
 
     my $t = $e->transport;
 
+    $t->max_requests(5)             # refresh_servers every 5 requests
     $t->protocol                    # eg 'http'
     $t->next_server                 # next node to use
     $t->current_server              # eg '127.0.0.1:9200' ie last used node
@@ -409,8 +444,8 @@ and will probably become the default after more testing in production.
 Note: my experience with L<HTTP::Lite> so far has been flawless - I'm just
 being cautious.
 
-See also: L<http://www.elasticsearch.com/docs/elasticsearch/modules/http>
-and L<http://www.elasticsearch.com/docs/elasticsearch/modules/thrift>
+See also: L<http://www.elasticsearch.org/guide/reference/modules/http.html>
+and L<http://www.elasticsearch.org/guide/reference/modules/thrift.html>
 
 =head1 SUBCLASSING TRANSPORT
 
@@ -487,7 +522,7 @@ You can register your Transport backend as follows:
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2010 Clinton Gormley.
+Copyright 2010 - 2011 Clinton Gormley.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
